@@ -45,16 +45,18 @@
 //! $ cxxbridge src/main.rs > path/to/mybridge.cc
 //! ```
 
+#![doc(html_root_url = "https://docs.rs/cxx-build/1.0.111")]
 #![allow(
     clippy::cast_sign_loss,
     clippy::default_trait_access,
+    clippy::derive_partial_eq_without_eq,
     clippy::doc_markdown,
-    clippy::drop_copy,
     clippy::enum_glob_use,
+    clippy::explicit_auto_deref,
     clippy::if_same_then_else,
     clippy::inherent_to_string,
+    clippy::into_iter_without_iter,
     clippy::items_after_statements,
-    clippy::let_underscore_drop,
     clippy::match_bool,
     clippy::match_on_vec_items,
     clippy::match_same_arms,
@@ -67,12 +69,15 @@
     clippy::or_fun_call,
     clippy::redundant_else,
     clippy::shadow_unrelated,
+    clippy::significant_drop_in_scrutinee,
     clippy::similar_names,
     clippy::single_match_else,
     clippy::struct_excessive_bools,
+    clippy::struct_field_names,
     clippy::too_many_arguments,
     clippy::too_many_lines,
     clippy::toplevel_ref_arg,
+    clippy::uninlined_format_args,
     clippy::upper_case_acronyms,
     // clippy bug: https://github.com/rust-lang/rust-clippy/issues/6983
     clippy::wrong_self_convention
@@ -366,7 +371,7 @@ fn make_crate_dir(prj: &Project) -> PathBuf {
     let crate_dir = prj.out_dir.join("cxxbridge").join("crate");
     let ref link = crate_dir.join(&prj.include_prefix);
     let ref manifest_dir = prj.manifest_dir;
-    if out::symlink_dir(manifest_dir, link).is_err() && cfg!(not(unix)) {
+    if out::relative_symlink_dir(manifest_dir, link).is_err() && cfg!(not(unix)) {
         let cachedir_tag = "\
         Signature: 8a477f597d28d172789f06886806bc55\n\
         # This file is a cache directory tag created by cxx.\n\
@@ -383,11 +388,11 @@ fn make_include_dir(prj: &Project) -> Result<PathBuf> {
     let cxx_h = include_dir.join("rust").join("cxx.h");
     let ref shared_cxx_h = prj.shared_dir.join("rust").join("cxx.h");
     if let Some(ref original) = env::var_os("DEP_CXXBRIDGE1_HEADER") {
-        out::symlink_file(original, cxx_h)?;
-        out::symlink_file(original, shared_cxx_h)?;
+        out::absolute_symlink_file(original, cxx_h)?;
+        out::absolute_symlink_file(original, shared_cxx_h)?;
     } else {
         out::write(shared_cxx_h, gen::include::HEADER.as_bytes())?;
-        out::symlink_file(shared_cxx_h, cxx_h)?;
+        out::relative_symlink_file(shared_cxx_h, cxx_h)?;
     }
     Ok(include_dir)
 }
@@ -396,6 +401,7 @@ fn generate_bridge(prj: &Project, build: &mut Build, rust_source_file: &Path) ->
     let opt = Opt {
         allow_dot_includes: false,
         cfg_evaluator: Box::new(CargoEnvCfgEvaluator),
+        doxygen: CFG.doxygen,
         ..Opt::default()
     };
     let generated = gen::generate_from_path(rust_source_file, &opt);
@@ -410,7 +416,7 @@ fn generate_bridge(prj: &Project, build: &mut Build, rust_source_file: &Path) ->
     out::write(header_path, &generated.header)?;
 
     let ref link_path = include_dir.join(rel_path);
-    let _ = out::symlink_file(header_path, link_path);
+    let _ = out::relative_symlink_file(header_path, link_path);
 
     let ref rel_path_cc = rel_path.with_appended_extension(".cc");
     let ref implementation_path = sources_dir.join(rel_path_cc);
@@ -419,8 +425,8 @@ fn generate_bridge(prj: &Project, build: &mut Build, rust_source_file: &Path) ->
 
     let shared_h = prj.shared_dir.join(&prj.include_prefix).join(rel_path_h);
     let shared_cc = prj.shared_dir.join(&prj.include_prefix).join(rel_path_cc);
-    let _ = out::symlink_file(header_path, shared_h);
-    let _ = out::symlink_file(implementation_path, shared_cc);
+    let _ = out::relative_symlink_file(header_path, shared_h);
+    let _ = out::relative_symlink_file(implementation_path, shared_cc);
     Ok(())
 }
 
@@ -451,7 +457,7 @@ fn best_effort_copy_headers(src: &Path, dst: &Path, max_depth: usize) {
             Ok(file_type) if file_type.is_file() => {
                 let src = entry.path();
                 match src.extension().and_then(OsStr::to_str) {
-                    Some("h") | Some("hh") | Some("hpp") => {}
+                    Some("h" | "hh" | "hpp") => {}
                     _ => continue,
                 }
                 if !dst_created && fs::create_dir_all(dst).is_err() {
